////////////////////////////////////////////////////////////////////////////////
// Fichier: GC-Softs/GCS.FaceBook.Client.pas
// Projet: GCS-FaceBook v0.1 - Copyright (c) 2010 GNU GPL v3
// Auteur: Guillaume Couture <GC-Softs@hotmail.com>
// SiteWeb: http://GcSofts.TuxFamily.org
////////////////////////////////////////////////////////////////////////////////

Unit GCS.FaceBook.Client;

INTERFACE

Uses
	Classes, SysUtils, IdHTTP, IdSSLOpenSSL,
	ulkJSON, // Utilisation de GCS.JSON plus tard...
	GCS.FaceBook.Exception;

Const
	cFACEBOOK_SUCCESSURL = 'http://www.facebook.com/connect/login_success.html';

Type
	////////////////////////////////////////
	// Classe GCS-FaceBook Client
	TGcsFaceBook_Client = Class(TObject)
	Protected
		FAppID: String;
		FAccess_Token: String;
		FExpires_In: String;
		FMetaData: Boolean;

		FIdHttp: TIdHTTP;
		FOpenSSL: TIdSSLIOHandlerSocketOpenSSL;

	Public
		Constructor Create(AAppID: String);
		Destructor Destroy; override;

		Function Login: Boolean;
		Function Logout: Boolean;
		Function LoginStatus: String;

		Function Graph(APath: String = ''; AParams: TStringList = NIL): TlkJSONobject;
		Function Request(AURL: String): TlkJSONobject;

		Function GetLoginURL(): String;
		Function GetLogoutURL(): String;
		Function GetLoginStatusURL(): String;
		Function GetURL(ASubDomain: String; APath: String = ''; AParams: TStringList = NIL): String;

	Public
		Property AppID: String Read FAppID Write FAppID;
		Property Access_Token: String Read FAccess_Token Write FAccess_Token;
		Property Expires_In: String Read FExpires_In Write FExpires_In;
		Property MetaData: Boolean Read FMetaData Write FMetaData;
	End;

IMPLEMENTATION

Uses
	Controls, IdURI,
	GCS.FaceBook.FormConnect,
	Dialogs;

////////////////////////////////////////
// Procedure Encode Request Params
// FIXME: A Mettre dans GCS.Network.URI plus Tard..
Procedure EncodeRequestParams(AStrings: TStringList);
Var I: Integer;
	LPos: Integer;
	LStr: String;
Begin
	Assert(AStrings <> NIL);

	For I := 0 to AStrings.Count - 1 Do Begin
		//AStrings[i] := AStrings.Names[i] + AStrings.NameValueSeparator + TIdURI.ParamsEncode(AStrings.ValueFromIndex[i]);
		LStr := AStrings[I];
		LPos := Pos('=', LStr);
		If (LPos > 0) Then AStrings[I] := Copy(LStr, 1, LPos-1) + '=' + TIdURI.ParamsEncode(Copy(LStr, LPos+1, MAXINT));
	End;
End;

////////////////////////////////////////
// Fonction Set Request Params
Function SetRequestParams(AStrings: TStringList): String;
Begin
	If Assigned(AStrings) Then Begin
		EncodeRequestParams(AStrings);

		If (AStrings.Count > 1) Then Begin
			// break trailing CR&LF
			Result := StringReplace(Trim(AStrings.Text), sLineBreak, '&', [rfReplaceAll]);
		End Else Begin
			Result := Trim(AStrings.Text);
		End;
	End Else Begin
		Result := '';
	End;
End;

////////////////////////////////////////
// Fonction Rcupration des Paramtres
// FIXME: A Mettre dans GCS.Network.URI plus Tard..
Function GetParams(AURL: String): String;
Begin
	Result := StringReplace(Trim(AURL), '#', sLineBreak, [rfReplaceAll]);
	Result := StringReplace(Trim(Result), '?', sLineBreak, [rfReplaceAll]);
	Result := StringReplace(Trim(Result), '&', sLineBreak, [rfReplaceAll]);
End;

////////////////////////////////////////
// Constructeur
Constructor TGcsFaceBook_Client.Create(AAppID: String);
Begin
	FAppID := AAppID;
	FIdHTTP := TIdHTTP.Create;

	FOpenSSL := TIdSSLIOHandlerSocketOpenSSL.Create;
	FOpenSSL.SSLOptions.Method := sslvSSLv3;

	FIdHTTP.IOHandler := FOpenSSL;
	FIdHTTP.ReadTimeout := 0;
	FIdHTTP.HandleRedirects := True;
	FIdHTTP.Request.ContentType := 'application/x-www-form-urlencoded';
	FIdHTTP.Request.UserAgent := 'GCS-FaceBook Client';
End;

////////////////////////////////////////
// Destructeur
Destructor TGcsFaceBook_Client.Destroy;
Begin
	FIdHTTP.Free;
	FOpenSSL.Free;
	Inherited Destroy;
End;

////////////////////////////////////////
// Fonction Login
Function TGcsFaceBook_Client.Login: Boolean;
Var
	FormConnect: TGcsFaceBook_FormConnect;
	Params: TStringList;
Begin
	Result := False;

	FormConnect := TGcsFaceBook_FormConnect.Create(NIL);
	Try
		Params := TStringList.Create;
		Try
			If FormConnect.Navigate(GetLoginURL(), cFACEBOOK_SUCCESSURL) = mrOK Then Begin
				Params.Text := GetParams(FormConnect.ResultURL);
				Access_Token := TIdURI.URLDecode(Params.Values['access_token']);
				Expires_In := Params.Values['expires_in'];
				Result := True;
			End;
		Finally
			Params.Free;
		End;
	Finally
		FormConnect.Free;
	End;
End;

////////////////////////////////////////
// Fonction Logout
Function TGcsFaceBook_Client.Logout: Boolean;
Begin
	FAccess_Token := '';
	FExpires_In   := '';

	// FIXME: Send Command Logout

	Result := True;
End;

////////////////////////////////////////
// Fonction Login Status
Function TGcsFaceBook_Client.LoginStatus: String;
Var
	FormConnect: TGcsFaceBook_FormConnect;
	Params: TStringList;
Begin
	Result := '';

	FormConnect := TGcsFaceBook_FormConnect.Create(NIL);
	Try
		Params := TStringList.Create;
		Try
			If FormConnect.Navigate(GetLoginStatusURL(), cFACEBOOK_SUCCESSURL) = mrOK Then Begin
				Params.Text := GetParams(FormConnect.ResultURL);
				Params.Delete(0);
				Result := TIdURI.URLDecode(Params.Text);
			End;
		Finally
			Params.Free;
		End;
	Finally
		FormConnect.Free;
	End;
End;

////////////////////////////////////////
// Fonction Graph
Function TGcsFaceBook_Client.Graph(APath: String = ''; AParams: TStringList = NIL): TlkJSONobject;
Var LParams: TStringList;
Begin
	LParams := TStringList.Create;
	Try
		If Assigned(AParams) Then LParams.Assign(AParams);

		// Ajout du Paramtre Access_Token
		If (LParams.Values['access_token'] = '') Then
			LParams.Values['access_token'] := FAccess_Token;

		// Ajout du Paramtre MetaData
		If (LParams.Values['metadata'] = '') Then
			If FMetaData Then LParams.Values['metadata'] := '1';

		// FIXME: Utilisation de TIdURI.URLEncode ou de TlkJSON.ParseText ???
		{
		// json_encode all params values that are not strings
		foreach (Params as Key => Value) Begin
			if (!is_string($value)) Then Begin
				params[$key] = json_encode($value);
			End;
		End;
		}

		Result := Request(GetURL('graph', APath, LParams));
	Finally
		LParams.Free;
	End;
End;

////////////////////////////////////////
// Fonction Request
Function TGcsFaceBook_Client.Request(AURL: String): TlkJSONobject;
Var ResultStr: String;
Begin
	Try ResultStr := FIdHttp.Get(AURL);
	Except
		On E: EIdHTTPProtocolException Do Begin
			If E.ErrorCode = 400 Then Begin
				Result := TlkJSON.ParseText(E.ErrorMessage) as TlkJSONobject;

				If Assigned(Result.Field['error']) And Assigned(Result.Field['error'].Field['type']) And (Result.Field['error'].Field['type'].Value = 'OAuthException') Then Begin
					FAccess_Token := '';
					FExpires_In := '';
				End;

				Raise EGcsFaceBook_Exception.Create(Result);
			End;

			Result := TlkJSONobject.Create();
			Result.Field['error_code'].Value := E.ErrorCode;
			Result.Field['error'].Field['message'].Value := E.ErrorMessage;
			Result.Field['error'].Field['type'].Value := 'IndyException';
			Raise EGcsFaceBook_Exception.Create(Result);
		End;
	End;

	Result := TlkJSON.ParseText(ResultStr) as TlkJSONobject;
End;

////////////////////////////////////////
// Fonction Get Login URL
// Paramtres  Vrifier: next, cancel_url, req_perms
Function TGcsFaceBook_Client.GetLoginURL(): String;
Var Params: TStringList;
Begin
	Params := TStringList.Create;
	Try
		Params.Values['client_id'] := FAppID;
		Params.Values['display'] := 'popup';
		Params.Values['type'] := 'user_agent';
		Params.Values['redirect_uri'] := cFACEBOOK_SUCCESSURL;

		Result := GetURL('graph', 'oauth/authorize', Params);
	Finally
		Params.Free;
	End;
End;

////////////////////////////////////////
// Fonction Get Logout URL
// FIXME: Est-ce la bonne faon pour ce dconnect ???
// Paramtres  Vrifier: Next = Success Logout
Function TGcsFaceBook_Client.GetLogoutURL(): String;
Var Params: TStringList;
Begin
	Params := TStringList.Create;
	Try
		Params.Values['access_token'] := FAccess_Token;
		Result := GetURL('www', 'logout.php', Params);
	Finally
		Params.Free;
	End;
End;

////////////////////////////////////////
// Fonction Get Login Status
// Paramtres  Vrifier: ok_session, no_session, no_user
Function TGcsFaceBook_Client.GetLoginStatusURL(): String;
Var Params: TStringList;
Begin
	Params := TStringList.Create;
	Try
		Params.Values['api_key'] := FAppID;
		Params.Values['session_version'] := '3';
		Params.Values['ok_session'] := TIdURI.URLEncode(cFACEBOOK_SUCCESSURL);
		Result := GetURL('www', 'extern/login_status.php', Params);
	Finally
		Params.Free;
	End;
End;

////////////////////////////////////////
// Fonction Get URL
// FIXME: Utilisation future de TIdURI...
Function TGcsFaceBook_Client.GetURL(ASubDomain: String; APath: String = ''; AParams: TStringList = NIL): String;
Begin
	Result := 'https://' + ASubDomain + '.facebook.com/' + APath;
	If Assigned(AParams) And (AParams.Count > 0) Then Result := Result + '?' + SetRequestParams(AParams);
End;

End.
